@@ -26,6 +26,7 @@
 
 CU8* help =
 "zxcall  -  Web Service Client tool R" ZXID_REL "\n\
+SAML 2.0 and ID-WSF 2.0 are standards for federated identity and web services.\n\
 Copyright (c) 2010 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.\n\
 NO WARRANTY, not even implied warranties. Licensed under Apache License v2.0\n\
 See http://www.apache.org/licenses/LICENSE-2.0\n\
@@ -46,18 +47,22 @@ Usage: zxcall [options] -s SESID -t SVCTYPE <soap_req_body.xml >soap_resp.xml\n\
   -az AZCREDS      Optional authorization credentials. Query string format.\n\
                    N.B. For authorization to work PDP_URL configuration option is needed.\n\
   -e SOAPBODY      Pass SOAP body as argument (default is to read from STDIN)\n\
+  -b               In response, only return content of SOAP body, omitting Envelope and Body.\n\
   -n               Dryrun. Do not actually make call. Instead print it to stdout.\n\
+  -l               List EPR cache (you need to specify -s SEDID or -a as well)\n\
   -v               Verbose messages.\n\
   -q               Be extra quiet.\n\
   -d               Turn on debugging.\n\
+  -dc              Dump config.\n\
   -h               This help message\n\
   --               End of options\n\
 \n\
-echo '<query>Foo</query>' | zxcall -a user:pw -t urn:x-demo-svc\n\
+echo '<query>Foo</query>' | zxcall -a https://idp.tas3.eu/zxididp?o=B user:pw -t urn:x-demo-svc\n\
 \n";
 
 int dryrun  = 0;
 int verbose = 1;
+int out_fmt = 0;
 char* entid = 0;
 char* idp   = 0;
 char* user  = 0;
@@ -67,11 +72,12 @@ char* url = 0;
 char* di  = 0;
 char* az  = 0;
 char* bdy = 0;
-struct zxid_conf* cf;
+zxid_conf* cf;
 
-/* Called by:  main x9 */
+/* Called by:  main x8, zxcall_main, zxcot_main */
 static void opt(int* argc, char*** argv, char*** env)
 {
+  struct zx_str* ss;
   if (*argc <= 1) return;
   
   while (1) {
@@ -102,6 +108,14 @@ static void opt(int* argc, char*** argv, char*** env)
       }
       break;
 
+    case 'b':
+      switch ((*argv)[0][2]) {
+      case '\0':
+	++out_fmt;
+	continue;
+      }
+      break;
+
     case 'c':
       switch ((*argv)[0][2]) {
       case '\0':
@@ -122,6 +136,10 @@ static void opt(int* argc, char*** argv, char*** env)
 	if ((*argc) < 1) break;
 	di = (*argv)[0];
 	continue;
+      case 'c':
+	ss = zxid_show_conf(cf);
+	fprintf(stderr, "\n======== CONF ========\n%.*s\n^^^^^^^^ CONF ^^^^^^^^\n",ss->len,ss->s);
+	continue;
       }
       break;
 
@@ -234,11 +252,11 @@ int zxcall_main(int argc, char** argv, char** env)
   int siz, got, n;
   char* p;
   struct zx_str* ss;
-  struct zxid_ses* ses;
-  struct zxid_entity* idp_meta;
+  zxid_ses* ses;
+  zxid_entity* idp_meta;
   
   strncpy(zx_instance, "\tzxcall", sizeof(zx_instance));
-  cf = zxid_new_conf(ZXID_PATH);
+  cf = zxid_new_conf_to_cf(0);
   opt(&argc, &argv, &env);
   
   if (sid) {
@@ -300,7 +318,11 @@ int zxcall_main(int argc, char** argv, char** env)
     }
     if (verbose)
       fprintf(stderr, "Call returned %d bytes.\n", ss->len);
-    printf("%.*s", ss->len, ss->s);
+    if (out_fmt) {
+      p = zxid_extract_body(cf, ss->s);
+      printf("%s", p);
+    } else
+      printf("%.*s", ss->len, ss->s);
   } else {
     D("Call Az(%s)", az);
     if (dryrun) {
