@@ -1,4 +1,5 @@
 /* zxidmkwsf.c  -  Handwritten nitty-gritty functions for constructing various elems
+ * Copyright (c) 2010 Sampo Kellomaki <sampo@iki.fi>, All Rights Reserved.
  * Copyright (c) 2007-2009 Symlabs (symlabs@symlabs.com), All Rights Reserved.
  * Author: Sampo Kellomaki (sampo@iki.fi)
  * This is confidential unpublished proprietary source code of the author.
@@ -10,6 +11,7 @@
  * 12.1.2007, created --Sampo
  * 7.10.2008, added documentation --Sampo
  * 15.11.2009, added ID-WSF <lu:Status> --Sampo
+ * 25.5.2010, added SOAP fault, tas3:Status, error formatting --Sampo
  */
 
 #include "errmac.h"
@@ -26,13 +28,21 @@
  * service. All other complicated options and multi service queries
  * will come in later releases. */
 
-/*() Create ID-WSF protocol <lu:Status> element, given various levels of error input. */
+/*() Create ID-WSF protocol <lu:Status> element, given various levels of error input.
+ *
+ * sc1:: First level status code
+ * sc2:: Second level status code, if any
+ * msg:: First level status message
+ * ref:: First level Status/@ref. Ref is used to point to culprit XML element, if any.
+ * returns:: lu:Status data structure with fields populated.
+ *
+ * See also: zxid_mk_fault() */
 
-/* Called by:  zxid_di_query x4, zxid_idp_as_do x3, zxid_mk_lu_Status */
-struct zx_lu_Status_s* zxid_mk_lu_Status(struct zxid_conf* cf, char* sc1, char* sc2, char* msg, char* ref)
+/* Called by:  zxid_di_query x4, zxid_idp_as_do x3, zxid_mk_fault, zxid_mk_lu_Status, zxid_mk_tas3_Status */
+struct zx_lu_Status_s* zxid_mk_lu_Status(zxid_conf* cf, const char* sc1, const char* sc2, const char* msg, const char* ref)
 {
   struct zx_lu_Status_s* st = zx_NEW_lu_Status(cf->ctx);
-  st->code = zx_dup_str(cf->ctx, sc1);
+  st->code = zx_dup_str(cf->ctx, STRNULLCHKQ(sc1));
   if (msg)
     st->comment = zx_dup_str(cf->ctx, msg);
   if (ref)
@@ -42,10 +52,157 @@ struct zx_lu_Status_s* zxid_mk_lu_Status(struct zxid_conf* cf, char* sc1, char*
   return st;
 }
 
+/*() Create TAS3 application level Status (error) header. */
+
+/* Called by: */
+zxid_tas3_status* zxid_mk_tas3_status(zxid_conf* cf, const char* ctlpt, const char* sc1, const char* sc2, const char* msg, const char* ref)
+{
+  zxid_tas3_status* st = zx_NEW_tas3_Status(cf->ctx);
+  if (ctlpt)
+    st->ctlpt = zx_dup_str(cf->ctx, ctlpt);
+  st->code = zx_dup_str(cf->ctx, STRNULLCHKQ(sc1));
+  if (msg)
+    st->comment = zx_dup_str(cf->ctx, msg);
+  if (ref)
+    st->ref = zx_dup_str(cf->ctx, ref);
+  if (sc2)
+    st->Status = zxid_mk_lu_Status(cf, sc2, 0, 0, 0);
+  st->mustUnderstand = zx_dup_str(cf->ctx, "0");
+  return st;
+}
+
+/*() Create SOAP Fault element (see Table 2 of [SOAPBind2], pp.12-13)
+ *
+ * fa:: Optional fault actor, such as one of the TAS3 control points (ctlpt)
+ * fc:: Fault code. Should be "e:Client" or "e:Server".
+ * fs:: Fault string. Human readable string explanation of the fault.
+ * sc1:: First level status code (to be placed inside <detail> element)
+ * sc2:: Second level status code, if any
+ * msg:: First level status message
+ * ref:: First level Status/@ref. Ref is used to point to culprit XML element, if any.
+ * returns:: Fault data structure with fields populated.
+ *
+ * See also: zxid_mk_lu_Status()
+ */
+
+/* Called by:  zxid_wsf_validate_a7n x5, zxid_wsp_validate x11 */
+struct zx_e_Fault_s* zxid_mk_fault(zxid_conf* cf, const char* fa, const char* fc, const char* fs, const char* sc1, const char* sc2, const char* msg, const char* ref)
+{
+  struct zx_e_Fault_s* flt = zx_NEW_e_Fault(cf->ctx);
+  if (fa)
+    flt->faultactor = zx_dup_simple_elem(cf->ctx, fa);
+  flt->faultcode = zx_dup_simple_elem(cf->ctx, fc?fc:"e:Client");
+  if (fs)
+    flt->faultstring = zx_dup_simple_elem(cf->ctx, fs);
+  if (sc1) {
+    flt->detail = zx_NEW_e_detail(cf->ctx);
+    flt->detail->Status = zxid_mk_lu_Status(cf, sc1, sc2, msg, ref);
+  }
+  return flt;
+}
+
+/*() Set current fault of the session. If current fault is set, the zxid_wsp_decorate()
+ * function will generate a SOAP Fault response instead of normal SOAP response. If
+ * you wish to return application response in situation where fault has been
+ * detected, you can use this function to reset the current fault to null. */
+
+/* Called by:  zxid_wsf_validate_a7n x5, zxid_wsp_validate x12 */
+void zxid_set_fault(zxid_conf* cf, zxid_ses* ses, struct zx_e_Fault_s* flt) {
+  if (ses->curflt) /* Free the previous fault */
+    zx_FREE_e_Fault(cf->ctx, ses->curflt, 1);
+  ses->curflt = flt;
+}
+
+/*() Read current fault of the session. NULL return means that there was no fault. */
+
+/* Called by: */
+struct zx_e_Fault_s* zxid_get_fault(zxid_conf* cf, zxid_ses* ses) {
+  return ses->curflt;
+}
+
+/* Called by: */
+char* zxid_get_tas3_fault_sc1(zxid_conf* cf, struct zx_e_Fault_s* flt) {
+  if (!flt || !flt->faultcode || !flt->faultcode->content || !flt->faultcode->content->s)
+    return 0;
+  return zx_str_to_c(cf->ctx, flt->faultcode->content);
+}
+/* Called by: */
+char* zxid_get_tas3_fault_sc2(zxid_conf* cf, struct zx_e_Fault_s* flt) {
+  if (!flt || !flt->detail || !flt->detail->Status || !flt->detail->Status->code || !flt->detail->Status->code->s)
+    return 0;
+  return zx_str_to_c(cf->ctx, flt->detail->Status->code);
+}
+/* Called by: */
+char* zxid_get_tas3_fault_comment(zxid_conf* cf, struct zx_e_Fault_s* flt) {
+  if (!flt || !flt->faultstring || !flt->faultstring->content || !flt->faultstring->content->s)
+    return 0;
+  return zx_str_to_c(cf->ctx, flt->faultstring->content);
+}
+/* Called by: */
+char* zxid_get_tas3_fault_ref(zxid_conf* cf, struct zx_e_Fault_s* flt) {
+  if (!flt || !flt->detail || !flt->detail->Status || !flt->detail->Status->ref || !flt->detail->Status->ref->s)
+    return 0;
+  return zx_str_to_c(cf->ctx, flt->detail->Status->ref);
+}
+/* Called by: */
+char* zxid_get_tas3_fault_actor(zxid_conf* cf, struct zx_e_Fault_s* flt) {
+  if (!flt || !flt->faultactor || !flt->faultactor->content || !flt->faultactor->content->s)
+    return 0;
+  return zx_str_to_c(cf->ctx, flt->faultactor->content);
+}
+
+/*() Set current TAS3 Status of the session. If current Status is set, the zxid_wsp_decorate()
+ * function will generate a TAS3 status header. */
+
+/* Called by:  zxid_wsp_validate */
+void zxid_set_tas3_status(zxid_conf* cf, zxid_ses* ses, zxid_tas3_status* status) {
+  if (ses->curstatus) /* Free the previous fault */
+    zx_FREE_tas3_Status(cf->ctx, ses->curstatus, 1);
+  ses->curstatus = status;
+}
+
+/*() Read current fault of the session. NULL return means that there was no fault. */
+
+/* Called by: */
+zxid_tas3_status* zxid_get_tas3_status(zxid_conf* cf, zxid_ses* ses) {
+  return ses->curstatus;
+}
+
+/* Called by: */
+char* zxid_get_tas3_status_sc1(zxid_conf* cf, zxid_tas3_status* st) {
+  if (!st || !st->code || !st->code->s)
+    return 0;
+  return zx_str_to_c(cf->ctx, st->code);
+}
+/* Called by: */
+char* zxid_get_tas3_status_sc2(zxid_conf* cf, zxid_tas3_status* st) {
+  if (!st || !st->Status || !st->Status->code || !st->Status->code->s)
+    return 0;
+  return zx_str_to_c(cf->ctx, st->Status->code);
+}
+/* Called by: */
+char* zxid_get_tas3_status_comment(zxid_conf* cf, zxid_tas3_status* st) {
+  if (!st || !st->comment || !st->comment->s)
+    return 0;
+  return zx_str_to_c(cf->ctx, st->comment);
+}
+/* Called by: */
+char* zxid_get_tas3_status_ref(zxid_conf* cf, zxid_tas3_status* st) {
+  if (!st || !st->ref || !st->ref->s)
+    return 0;
+  return zx_str_to_c(cf->ctx, st->ref);
+}
+/* Called by: */
+char* zxid_get_tas3_status_ctlpt(zxid_conf* cf, zxid_tas3_status* st) {
+  if (!st || !st->ctlpt || !st->ctlpt->s)
+    return 0;
+  return zx_str_to_c(cf->ctx, st->ctlpt);
+}
+
 /*() Low level constructor for discovery <di:RequestedService>. */
 
 /* Called by:  zxid_mk_di_query */
-static struct zx_di_RequestedService_s* zxid_mk_di_req_svc(struct zxid_conf* cf, int req_id, const char* svc_type, const char* url, const char* di_opt, const char* action)
+static struct zx_di_RequestedService_s* zxid_mk_di_req_svc(zxid_conf* cf, int req_id, const char* svc_type, const char* url, const char* di_opt, const char* action)
 {
   struct zx_di_RequestedService_s* rs = zx_NEW_di_RequestedService(cf->ctx);
 #if 0
@@ -74,8 +231,8 @@ static struct zx_di_RequestedService_s* zxid_mk_di_req_svc(struct zxid_conf* cf,
 
 /*() Low level constructor for discovery <di:Query>. */
 
-/* Called by:  main x3, zxid_get_epr */
-struct zx_di_Query_s* zxid_mk_di_query(struct zxid_conf* cf, const char* svc_type, const char* url, const char* di_opt, const char* action)
+/* Called by:  main x2, zxid_get_epr */
+struct zx_di_Query_s* zxid_mk_di_query(zxid_conf* cf, const char* svc_type, const char* url, const char* di_opt, const char* action)
 {
   struct zx_di_Query_s* q = zx_NEW_di_Query(cf->ctx);
   q->RequestedService = zxid_mk_di_req_svc(cf, 1, svc_type, url, di_opt, action);
@@ -84,8 +241,8 @@ struct zx_di_Query_s* zxid_mk_di_query(struct zxid_conf* cf, const char* svc_typ
 
 /*() Low level constructor for WSA <Address>. */
 
-/* Called by:  zxid_wsc_call x2 */
-struct zx_a_Address_s* zxid_mk_addr(struct zxid_conf* cf, struct zx_str* url)
+/* Called by:  zxid_wsc_prep, zxid_wsf_decor */
+struct zx_a_Address_s* zxid_mk_addr(zxid_conf* cf, struct zx_str* url)
 {
   struct zx_a_Address_s* addr = zx_NEW_a_Address(cf->ctx);
   addr->gg.content = url;
@@ -96,8 +253,8 @@ struct zx_a_Address_s* zxid_mk_addr(struct zxid_conf* cf, struct zx_str* url)
 
 /*() Low level constructor for <dap:Select>. */
 
-/* Called by:  main x8 */
-struct zx_dap_Select_s* zxid_mk_dap_select(struct zxid_conf* cf, char* dn, char* filter, char* attributes, int derefaliases, int scope, int sizelimit, int timelimit, int typesonly)
+/* Called by:  main x4 */
+struct zx_dap_Select_s* zxid_mk_dap_select(zxid_conf* cf, char* dn, char* filter, char* attributes, int derefaliases, int scope, int sizelimit, int timelimit, int typesonly)
 {
   struct zx_dap_Select_s* sel = zx_NEW_dap_Select(cf->ctx);
   if (dn)           sel->dn = zx_ref_simple_elem(cf->ctx, dn);
@@ -113,8 +270,8 @@ struct zx_dap_Select_s* zxid_mk_dap_select(struct zxid_conf* cf, char* dn, char*
 
 /*() Low level constructor for <dap:QueryItem>. */
 
-/* Called by:  main x6 */
-struct zx_dap_QueryItem_s* zxid_mk_dap_query_item(struct zxid_conf* cf, struct zx_dap_Select_s* sel, char* objtype, char* predef, char* sort, char* changed_since, int incl_common_attr, int offset, int count, char* setreq, char* setid, char* contingent_itemidref)
+/* Called by:  main x3 */
+struct zx_dap_QueryItem_s* zxid_mk_dap_query_item(zxid_conf* cf, struct zx_dap_Select_s* sel, char* objtype, char* predef, char* sort, char* changed_since, int incl_common_attr, int offset, int count, char* setreq, char* setid, char* contingent_itemidref)
 {
   struct zx_dap_QueryItem_s* qi = zx_NEW_dap_QueryItem(cf->ctx);
   qi->Select = sel;
@@ -156,8 +313,8 @@ struct zx_dap_QueryItem_s* zxid_mk_dap_query_item(struct zxid_conf* cf, struct z
 
 /*() Low level constructor for <dap:TestOp>. */
 
-/* Called by:  main x2 */
-struct zx_dap_TestOp_s* zxid_mk_dap_testop(struct zxid_conf* cf, char* dn, char* filter, char* attributes, int derefaliases, int scope, int sizelimit, int timelimit, int typesonly)
+/* Called by:  main */
+struct zx_dap_TestOp_s* zxid_mk_dap_testop(zxid_conf* cf, char* dn, char* filter, char* attributes, int derefaliases, int scope, int sizelimit, int timelimit, int typesonly)
 {
   struct zx_dap_TestOp_s* sel = zx_NEW_dap_TestOp(cf->ctx);
   if (dn)           sel->dn = zx_ref_simple_elem(cf->ctx, dn);
@@ -173,8 +330,8 @@ struct zx_dap_TestOp_s* zxid_mk_dap_testop(struct zxid_conf* cf, char* dn, char*
 
 /*() Low level constructor for <dap:TestItem>. */
 
-/* Called by:  main x2 */
-struct zx_dap_TestItem_s* zxid_mk_dap_test_item(struct zxid_conf* cf, struct zx_dap_TestOp_s* top, char* objtype, char* predef)
+/* Called by:  main */
+struct zx_dap_TestItem_s* zxid_mk_dap_test_item(zxid_conf* cf, struct zx_dap_TestOp_s* top, char* objtype, char* predef)
 {
   struct zx_dap_TestItem_s* ti = zx_NEW_dap_TestItem(cf->ctx);
   ti->TestOp = top;
@@ -188,8 +345,8 @@ struct zx_dap_TestItem_s* zxid_mk_dap_test_item(struct zxid_conf* cf, struct zx_
 
 /*() Low level constructor for <dap:ResultQuery>. */
 
-/* Called by:  main x2 */
-struct zx_dap_ResultQuery_s* zxid_mk_dap_resquery(struct zxid_conf* cf, struct zx_dap_Select_s* sel, char* objtype, char* predef, char* sort, char* changed_since, int incl_common_attr, char* contingent_itemidref)
+/* Called by:  main */
+struct zx_dap_ResultQuery_s* zxid_mk_dap_resquery(zxid_conf* cf, struct zx_dap_Select_s* sel, char* objtype, char* predef, char* sort, char* changed_since, int incl_common_attr, char* contingent_itemidref)
 {
   struct zx_dap_ResultQuery_s* qi = zx_NEW_dap_ResultQuery(cf->ctx);
   qi->Select = sel;
@@ -224,8 +381,8 @@ struct zx_dap_ResultQuery_s* zxid_mk_dap_resquery(struct zxid_conf* cf, struct z
 
 /*() Low level constructor for <dap:Subscription>. */
 
-/* Called by:  main x2 */
-struct zx_dap_Subscription_s* zxid_mk_dap_subscription(struct zxid_conf* cf, char* subsID, char* itemidref, struct zx_dap_ResultQuery_s* rq, char* aggreg, char* trig, char* starts, char* expires, int incl_data, char* admin_notif, char* notify_ref)
+/* Called by:  main */
+struct zx_dap_Subscription_s* zxid_mk_dap_subscription(zxid_conf* cf, char* subsID, char* itemidref, struct zx_dap_ResultQuery_s* rq, char* aggreg, char* trig, char* starts, char* expires, int incl_data, char* admin_notif, char* notify_ref)
 {
   struct zx_dap_Subscription_s* subs = zx_NEW_dap_Subscription(cf->ctx);
   if (itemidref) {
@@ -250,8 +407,8 @@ struct zx_dap_Subscription_s* zxid_mk_dap_subscription(struct zxid_conf* cf, cha
 
 /*() Low level constructor for <dap:Query>. */
 
-/* Called by:  main x6 */
-struct zx_dap_Query_s* zxid_mk_dap_query(struct zxid_conf* cf, struct zx_dap_TestItem_s* tis, struct zx_dap_QueryItem_s* qis, struct zx_dap_Subscription_s* subs)
+/* Called by:  main x3 */
+struct zx_dap_Query_s* zxid_mk_dap_query(zxid_conf* cf, struct zx_dap_TestItem_s* tis, struct zx_dap_QueryItem_s* qis, struct zx_dap_Subscription_s* subs)
 {
   struct zx_dap_Query_s* q = zx_NEW_dap_Query(cf->ctx);
   q->TestItem = tis;
